searchData={"items":[{"type":"module","title":"XmlToMap","doc":"Simple convenience module for getting a map out of an XML string.","ref":"XmlToMap.html"},{"type":"function","title":"XmlToMap.default_namespace_match_fn/0","doc":"","ref":"XmlToMap.html#default_namespace_match_fn/0"},{"type":"function","title":"XmlToMap.naive_map/2","doc":"`naive_map/1` utility is inspired by `Rails Hash.from_xml()` but is\n\"naive\" in that it is convenient (requires no setup) but carries the same\ndrawbacks.\n\nFor example no validation over what should be a collection.  If and only if\nnodes are repeated at the same level will they become a list.  If a node has\nattributes we'll prepend a \"-\" in front of them and merge them into the map\nand take the node value and nest that inside \"#content\" key.","ref":"XmlToMap.html#naive_map/2"},{"type":"function","title":"XmlToMap.nested_map/2","doc":"","ref":"XmlToMap.html#nested_map/2"},{"type":"module","title":"XmlToMap.NaiveMap","doc":"Module to recursively traverse the output of `erlsom.simple_form/1`\nand produce a map.\n\n`erlsom` uses character lists so this library converts them to\nElixir binary string.\n\nAttributes, if present, are defined as `tag@attribute_name => attribute_value`.","ref":"XmlToMap.NaiveMap.html"},{"type":"function","title":"XmlToMap.NaiveMap.parse/1","doc":"","ref":"XmlToMap.NaiveMap.html#parse/1"},{"type":"module","title":"XmlToMap.NestedMap","doc":"Module to recursively traverse the output of `erlsom.simple_form/1`\nand produce a nested map with `name`, `attributes`, and `content` fields.\n\n`erlsom` uses character lists so this library converts them to\nElixir binary string.","ref":"XmlToMap.NestedMap.html"},{"type":"function","title":"XmlToMap.NestedMap.parse/2","doc":"","ref":"XmlToMap.NestedMap.html#parse/2"},{"type":"type","title":"XmlToMap.NestedMap.element/0","doc":"Type of a nested map","ref":"XmlToMap.NestedMap.html#t:element/0"},{"type":"extras","title":"XmlToMap","doc":"# XmlToMap\n\n[![Module Version](https://img.shields.io/hexpm/v/elixir_xml_to_map.svg)](https://hex.pm/packages/elixir_xml_to_map)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/elixir_xml_to_map/)\n[![Total Download](https://img.shields.io/hexpm/dt/elixir_xml_to_map.svg)](https://hex.pm/packages/elixir_xml_to_map)\n[![License](https://img.shields.io/hexpm/l/elixir_xml_to_map.svg)](https://github.com/homanchou/elixir_xml_to_map/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/homanchou/elixir-xml-to-map.svg)](https://github.com/homanchou/elixir-xml-to-map/commits/master)\n\nThis library provides two functions to create an Elixir Map data structure from an XML string.\n\n### `XmlToMap.naive_map/2`\n\nUsage:\n\n```elixir\nXmlToMap.naive_map(\"  123  \")\n```\n\nResults in:\n\n```elixir\n%{\"foo\" => %{\"bar\" => \"123\"}}\n```\n\nConverts XML string to an Elixir map with strings for keys, not atoms, since atoms are not garbage collected.\n\nThis tool is inspired by Rails `Hash.from_xml()`.\n\n----\n\nI call the function \"naive\", because there are known short comings and there is some [controversy](https://stackoverflow.com/questions/40650482/how-to-convert-xml-to-a-map-in-elixir) around using a conversion tool like this since XML and Maps are non-isomorphic and there is no standard way to convert all the information from one format to another.  The recommended way to pull specific well structured information from XML is to use something like xpath.  But if you understand the risks and still prefer to convert the whole XML string to a map then this tool is for you!\n\nIt is currently not able to parse XML namespace.  It also can't determine if a child should be a list unless it sees a repeated child.  If and only if nodes are repeated at the same level will they become a list.\n\n```elixir\n# there are two points inside foo, so the value of \"point\" becomes a list. Had \"foo\" only contained one point then there would be no list but instead one nested map\nXmlToMap.naive_map(\"   1  5    2  9   \")\n\n# => %{\"foo\" => %{\"point\" => [%{\"x\" => \"1\", \"y\" => \"5\"}, %{\"x\" => \"2\", \"y\" => \"9\"}]}}\n```\n\nPreviously this package did not handle XML node attributes.\nThe current version takes inspiration from a [go goxml2json package](https://github.com/basgys/goxml2json) and exports attributes in the map while prepending \"-\" so you know they are attributes.\n\nWhenever we encounter an XML node with BOTH attributes and children, we wrap \"#content\" around the node's inner value.\n\nFor example this snippet has a `Height` leaf node with attribute `Units` and a value of `0.50`:\n\n```xml\n \n    0.50 \n \n```\n\nThis would become this snippet:\n\n```json\n...\n\"ItemDimensions\": {\n     \"Height\": {\n           \"#content\": \"0.50\",\n           \"-Units\": \"inches\"\n     }\n}\n```\n\nEmpty tags will have a value of nil.\n\n### `XmlToMap.nested_map/2`\n\nThis function produces arguably more verbose and less straightforward result, but it preserves\nthe order of sequences and might be then instantiated back to _XML_ in an isomorphic way. \n\nIn general, it reflects the tree structure of an input _XML_ per se.\n\nUsage:\n\n```elixir\nXmlToMap.nested_map(\"  123  \")\n```\n\nResults in:\n\n```elixir\n%{\n  attributes: [],\n  name: \"foo\",\n  content: %{attributes: [], name: \"bar\", content: \"123\"}\n}\n```\n\nThe function does not make any assumptions about the content and recursively builds the result.\n\n```xml\n \n    0.50 \n \n```\n\nThis would become this snippet:\n\n```elixir\n%{\n  attributes: [],\n  name: \"ItemDimensions\",\n  content: %{attributes: [{\"Units\", \"inches\"}], name: \"Height\", content: \"0.50\"}\n}\n```\n\nWhich might then be converted to _JSON_ of the same shape.\n\nTo make it less verbose, pass `purge_empty: true` as the second parameter to `XmlToMap.nested_map/2`:\n\n```elixir\nXmlToMap.nested_map(\"  123   \", purge_empty: true)\n```\n\nResults in:\n\n```elixir\n%{\n  name: \"foo\",\n  content: [\n    %{attributes: [{\"arg\", \"yes\"}], name: \"bar\", content: \"123\"},\n    %{name: \"baz\"}\n  ]\n}\n```\n\n-----\n\nThere is a dependency on Erlsom to parse XML then converts the 'simple_form' structure into a map.\n\nI prefer Erlsom because it is the best documented erlang XML parser and because it mentions that it does not produce new atoms during the scanning.\n\nSee tests for some example usage.\n\n----","ref":"readme.html"},{"type":"extras","title":"Installation - XmlToMap","doc":"The package can be installed as:\n\n1.  Add `:elixir_xml_to_map` to your list of dependencies in `mix.exs`:\n\n    ```elixir\n    def deps do\n      [{:elixir_xml_to_map, \"~> 2.0\"}]\n    end\n    ```\n\n2.  Ensure `:elixir_xml_to_map` is started before your application:\n\n    ```elixir\n    def application do\n      [extra_applications: [:elixir_xml_to_map]]\n    end\n    ```","ref":"readme.html#installation"},{"type":"extras","title":"License - XmlToMap","doc":"Copyright (c) 2016-present, Homan Chou\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"}],"content_type":"text/markdown"}